/*============================================================================
| 2008 -  David Duvivier
|
| Operating System Basic Definitions and Tools
#############################################################################
#This program is free software; you can redistribute it and/or modify       #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#############################################################################
============================================================================*/
#include "ddtools.h"

/*---------------------------- Global variables ----------------------------*/
volatile sig_atomic_t AbortedByTimeOut = 0; /* Timeout si passe  1 */

/*-------------------------- Local functions -------------------------------*/

/*-------------------------- Local variables -------------------------------*/
/* ATTENTION : CES VARIABLES SONT PROPRES A CHAQUE PROCESSUS ET NE PEUVENT  */
/* ETRE UTILISEE EN GUISE DE VARIABLES "PARTAGEES/GLOBALES" */
static int semid = -1; /* Identificateur du tableau de smaphores partag */
		       /* Commun au pre et aux fils */
static int NB_SEMAPHORES = -1; 	 /* Nombre de smaphores dans le tableau */
				 /* de smaphores identifi par semid */
				 /* Commun au pre et aux fils */
static int DDTOOLSInit = -1;	 /* DDTools correctement initialis ? */
				 /* (pour le processus courant) */

/*------------------------------ Constantes --------------------------------*/
const char project_id = 'd';

/*============================================================================
|                         Procdures et Fonctions
============================================================================*/

/*----------------------------------------------------------------------------
| Initialisation du gnrateur alatoire 
| ATTENTION doit tre effectu par chaque processus (pre, fils...)
----------------------------------------------------------------------------*/
void DD_initrandom(void) {
  time_t t;    /* Mesure du temps (si ncessaire) */

  /* Initialisations diverses si ncessaires pour la suite ... */
  t = time(NULL); /* Pas ncessaire, mais utilisable pour benchmarking */
  srand(t); /* Initialiser le gnrateur alatoire */
}

/* Acquisition de NB_SEMAPHORES smaphore(s) */
/* ATTENTION : le descripteur associ est retourn dans la variable */
/* statique "semid" */
/* Utilise le chemin d'accs au nom du programme pass via la variable "filename" */
/* pour calculer le hashcode associ au tableau de smaphores */
/* Le nombre de smaphores dans le tableau de smaphores est pass via la variable */
/* Nombre_Semaphores */
int DD_sem_init(char *filename, int Nombre_Semaphores) {
  key_t k;

  if (semid == -1) {
    /* Premier appel -> Accder au tableau de smaphores s'il existe dj dans la liste IPC */
    /* REM : en ce point comme semid = -1 -> NB_SEMAPHORES = -1 par construction */
    semid = semget(k = ftok(filename, project_id), Nombre_Semaphores, 0600);
    if (semid == -1) {
      /* Premier appel et tableau introuvable -> Cration d'un nouveau tableau de smaphores */
      semid = semget(k = ftok(filename, project_id), Nombre_Semaphores, 0600 | IPC_CREAT);
      if (semid <= -1) {
        /* Erreur lors de l'acquisition */
        perror("Impossible d'obtenir des smaphores");
        exit(EXIT_FAILURE);
      }
      else {
        /* OK : on a des smaphores ! */
        NB_SEMAPHORES = Nombre_Semaphores;
        fprintf(stderr, "PEREouFILS  :pid=%d:Tableau de smaphores allou correctement (semid=%d)\n", getpid(), semid);
        fflush(stderr);
        return (1); /* Signale nouveau tableau de smaphores  l'appelant */
      }
    }
    else {
      /* Le tableau de smaphores existe dj et c'est le premier appel  sem_init */
      /* -> En ce point l'accs au tableau de smaphores est accept donc le nombre de smaphores */
      /*    est inchang par rapport  la prcdente allocation -> OK ! */
      /* -> le rutiliser ! */
      NB_SEMAPHORES = Nombre_Semaphores;
      fprintf(stderr, "PEREouFILS  :pid=%d:Tableau de smaphores r-utilis (semid=%d)\n", getpid(), semid);
      fflush(stderr);
      return (0); /* Signale r-utilisation des smaphores  l'appelant */
    }
  }
  else {
    /* Le tableau de smaphores existe dj et ce n'est pas le premier appel  sem_init */
    /* -> le rutiliser s'il contient le mme nombre de smaphores ! */
    if (Nombre_Semaphores == NB_SEMAPHORES) {
      fprintf(stderr, "PEREouFILS  :pid=%d:Tableau de smaphores r-utilis (semid=%d)\n", getpid(), semid);
      fflush(stderr);
      return (-1); /* Signale r-utilisation des smaphores  l'appelant */
                   /* et appels multiples de sem_init */
    }
    else {
      /* Erreur : tentative de rutilisation d'un tableau de smaphore dj allou dans la liste IPC */
      /* avec un nombre de smaphores diffrents -> Fin du programme, c'est  l'utilisateur */
      /* de supprimer l'ancien tableau pour en crer un nouveau */
      /* Erreur lors de l'acquisition */
      DD_pidmsg2("PERE?", "Tableau de smaphores dj allou, mais de taille incorrecte");
      exit(EXIT_FAILURE);
    }
  }
}

/* Suppression du tableau de smaphores dont l'id est stock dans la*/
/* variable globale semid. semid est reinitialise a -1 par cette procedure */
void DD_sem_destroy(void) {
  semctl(semid, 0, IPC_RMID, 0);
  semid = -1; /* Scurit : indique smaphores non allou */
  NB_SEMAPHORES = -1; /* Idem */
}

/*----------------------------------------------------------------------------
| Fixe la valeur initiale du smaphore N"sem"  la valeur "val"
----------------------------------------------------------------------------*/
void DD_sem_setvalue(int sem, int val) {
#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
       /* l'union semun est dfinie par l'inclusion de <sys/sem.h> */
       union  semun arg;
#else
       /* d'aprs X/OPEN il faut la dfinir nous-mmes */
       union semun {
             int val;                  /* valeur pour SETVAL */
             struct semid_ds *buf;     /* buffer pour IPC_STAT, IPC_SET */
             unsigned short *array;    /* table  pour GETALL, SETALL */
                                       /* Spcificit Linux : */
             struct seminfo *__buf;    /* buffer pour IPC_INFO */
       } arg;
#endif

  arg.val = val; /* Fixe la valeur initiale du smaphore */
  if (semctl(semid, sem, SETVAL, arg) <= -1) {
    perror("Impossible de fixer la valeur initiale du smaphore");
    exit(EXIT_FAILURE);
  }
}

/*----------------------------------------------------------------------------
| Lit "la valeur" du smaphore numro "sem" du au tableau de smaphores
| ATTENTION : SEULES LES VALEURS >= 0 SONT RENVOYEES (i.e. s'il y a des
| processus bloqus par le smaphores, la valeur retroune sera "0" et non
| une valeur ngative relative au nombre de processus bloqus
----------------------------------------------------------------------------*/
int DD_sem_getvalue(int sem) {
  int val; /* "Valeur" du smaphore */
  if ((val = semctl(semid, sem, GETVAL)) <= -1) {
    perror("Impossible de lire la valeur du smaphore");
    exit(EXIT_FAILURE);
  }
  return (val); /* Retourne la valeur du smaphore */
}

/*----------------------------------------------------------------------------
| Demande l'accs  la ressource protge par le smaphore N"sem" :
| quivaut  P(sem)
| quivaut  Down(sem)
----------------------------------------------------------------------------*/
int DD_sem_askresource (int sem) {
  int r;
  struct sembuf s;
  s.sem_num = sem;
  s.sem_op = -1;
  s.sem_flg = 0;
  if ((r = semop(semid, &s, 1)) <= -1) {
    perror("Impossible de rserver la ressource");
    exit(EXIT_FAILURE);
  }
  return r;
}

/*----------------------------------------------------------------------------
| Libre l'accs  la ressource protge par le smaphore N"sem" :
| quivaut  V(sem)
| quivaut  Up(sem)
----------------------------------------------------------------------------*/
int DD_sem_releaseresource (int sem) {
  int r;
  struct sembuf s;
  s.sem_num = sem;
  s.sem_op = 1;
  s.sem_flg = 0;
  if ((r = semop(semid, &s, 1)) <= -1) {
    perror("Impossible de librer la ressource");
    exit(EXIT_FAILURE);
  }
  return r;
}

/*----------------------------------------------------------------------------
| Bloque le processus pre tant qu'au moins un fils est actif.
----------------------------------------------------------------------------*/
void DD_waitallchilds(void) {
  int filspid; /* Pid du fils qui vient de se terminer */
  int status; /* Dclaration locale pour obtenir le code de retour du fils */

  while ((filspid = waitpid(-1, &status, 0))) {
    if (filspid < 0) {
      /* Se produit galement si plus/pas de fils -> normal ! */
      /* --> Je n'affiche rien ! */
      break; /* Je casse simplement la boucle mais sans appel  exit */
    }
    /* Vrification de la bonne terminaison du fils */
    if WIFEXITED(status) {
      /* Le fils s'est termin normalement par exit ou return */
      fprintf(stderr, "%-12s:pid=%d:Fin normale du processus %d\n", "PERE?", getpid(), filspid);
    }
    else {
      /* Le fils s'est termin anormalement par un signal ou une exception */
      fprintf(stderr, "%-12s:pid=%d:Fin anormale du processus %d\n", "PERE?", getpid(), filspid);
    }
    fflush(stderr);
  }
}

/*----------------------------------------------------------------------------
| Initialisation des structures de donnes de DDTools
| ATTENTION doit tre effectue une seule fois par le processus pre
|           AVANT DE CREER LES PROCESSUS FILS via fork()
| Les paramtres sont les suivants :
| - argc, argv        : argc et argv de la procdure principale main()
| - Nombre_Semaphores : nombre de smaphores dans le tableau de smaphores
------------------------------------------------------------------------------
| ATTENTION : Le test sur DDTOOLSInit ci-dessous permet de garantir que 
| chaque processus appelle au plus une et une seule fois DD_init, mais en
| aucun cas il ne permet de verifier que DD_init a t appel une seule
| fois par tous les processus car DDTOOLSInit n'est pas une variable
| partage par tous les processus de l'application courante !!!
----------------------------------------------------------------------------*/
void DD_init(int argc, char *argv[], int Nombre_Semaphores) {
  if (DDTOOLSInit == -1) {
    /* Premire initialisation dans le processus courant -> OK ! */
    DD_sem_init(argv[0], Nombre_Semaphores);
    DD_initrandom(); /* Par scurit ! */
    DDTOOLSInit = 0; /* DDTools correctement initialis ! */
  }
  else {
    /* Erreur : DD_init doit tre appel une seule fois !!! */
    DD_pidmsg2("PEREouFILS", "ERREUR : DD_init() doit tre appel une seule fois !");
    exit(EXIT_FAILURE);
  }
}

/*----------------------------------------------------------------------------
| Libration des ressources systme utilises par DDTools
| ATTENTION doit tre effectue une seule fois par le dernier processus actif
|	    (i.e. classiquement le processus pre)
------------------------------------------------------------------------------
| ATTENTION : Le test sur DDTOOLSInit ci-dessous permet de garantir que 
| chaque processus appelle au plus une et une seule fois DD_close, mais en
| aucun cas il ne permet de verifier que DD_init a t appel une seule
| fois par tous les processus car DDTOOLSInit n'est pas une variable
| partage par tous les processus de l'application courante !!!
----------------------------------------------------------------------------*/
void DD_close(void) {
  if (DDTOOLSInit == 0) {
    /* Premier appel -> OK ! */
    DD_sem_destroy();
    DDTOOLSInit = -1;
  }
  else {
    /* Erreur : DD_close doit tre appel une seule fois !!! */
    DD_pidmsg2("PEREouFILS", "ERREUR : DD_close() doit tre appel une seule fois !");
    exit(EXIT_FAILURE);
  }
}
