# -*- coding: utf-8 -*-
#
# Copyright (C) 2010 Sébastien Dumont <sdumont@masteri2l.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Import from the Standard Library
from imp import load_module, PKG_DIRECTORY
from os import getcwd
from unittest import TestCase, main

# Import from here
load_module('clientedl', None, getcwd(), ('', '', PKG_DIRECTORY))
from clientedl.shell import Shell, ShellCommand
from clientedl.shell.exceptions import CommandError, CommandNotFoundError
from clientedl.shell.exceptions import TooManyArgError, TooFewArgError



class ShellTestCase(TestCase):

    def setUp(self):
        self.shell = Shell()
        self.command = ShellCommand()
        self.command.MIN_ARG = 0
        self.command.MAX_ARG = 0
        self.shell.command_list = {'command': self.command}


    def test_existant_command(self):
        command = 'command'
        arguments = []
        self.shell.run_command(command, arguments)
        #TODO use doctest? http://docs.python.org/lib/doctest-unittest-api.html
# CHECK


    def test_unexistant_command(self):
        command = 'unknowcommand'
        arguments = []
        self.assertRaises(CommandNotFoundError, self.shell.run_command, command, arguments)


    def test_too_many_arguments(self):
        self.command.MIN_ARG = 0
        self.command.MAX_ARG = 0
        command = 'command'
        arguments = ['blabla']
        self.assertRaises(TooManyArgError, self.shell.run_command, command, arguments)


    def test_too_many_arguments(self):
        self.command.MIN_ARG = 1
        self.command.MAX_ARG = 2
        command = 'command'
        arguments = []
        self.assertRaises(TooFewArgError, self.shell.run_command, command, arguments)


    #TODO check any kind of exceptions which can be raised
    #TODO check arguments validation


# class BasicCommandTestCase(TestCase):
#TODO check basic commands (quit, help)



if __name__ == '__main__':
    main()
