/*----------------------------------------------------------------------*/
/* Fichier : consommateursig.c						*/
/* Auteur  : D. Duvivier - LIL/ULCO					*/
/* Date    : 22/09/09							*/
/* Objectif: TP N3 (correction) M2/I2L Systme Rseaux Scurit	*/
/*	     Test de deux signaux entre deux processus "lourds"		*/
/*	     Ce programme est le consommateur de donnes.		*/
/*	     ===========================================		*/
/*	     Il ne doit pas tre lanc directement depuis le shell	*/
/*	     ====================================================	*/
/*	     (le producteur s'en charge ds qu'il est initialis/pret).	*/
/*----------------------------------------------------------------------*/
/* Remarque : les appels "fflush(stdout)" ne servent  rien si ce n'est	*/
/* pour s'assurer que les messages soient bien affichs immdiatement	*/
/* en vitant les dlais ds  la buffrisation des e/s.		*/
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>

/* Variables globales -> viter/limiter normalement (!) */

/* Gestion des signaux (la synchronisation est totalement gre par */
/* cette fonction */
void signal_handler(int signal) {
  static int iping = 0;
  switch(signal) {
  case SIGUSR1:
    printf("Consommateur (pid=%d) : Signal SIGUSR1 intercept !\n", getpid());
    sleep(1); /* Simule un traitement */
    iping++;
    printf("Consommateur (pid=%d) : Emet PONG %d via SIGUSR2 destin au producteur\n", getpid(), iping);
    fflush(stdout);

    kill(getppid(), SIGUSR2); /* Envoit le signal SIGUSR2 au pre */
  break;
  case SIGHUP:
    puts("Signal SIGHUP intercept par notre handler");
  break;
  }
}

int main(void) {
  int i = 0; /* Indice de boucle */

  printf("Consommateur (pid=%d) : Lancement du processus consommateur...\n", getpid());
  fflush(stdout);

  signal(SIGHUP, signal_handler);
  signal(SIGUSR1, signal_handler);

  /* Ci-dessous la boucle sur sleep() simule le traitement normal */
  /* du processus fils */
  /* Ce traitement "normal" est priodiquement interrompu par le pre */
  /* et signal_handler intercepte ce signal, traite le signal et */
  /* renvoit un signal au pre. */
  /* Ici le programme principal ne s'occupe absolument pas de la gestion*/
  /* des signaux (qui est totalement gre par signal_handler() */
  /* ceci est possible car il n'y a pas de variable partage (...) */
  /* entre signal_handler() et main(), sinon attention il faut grer */
  /* ces partages correctement via des sections critiques ... */
  for (i = 1; i <= 14; i++) {
    printf("Consommateur (pid=%d) : Boucle %d sur sleep()...\n", getpid(), i);
    fflush(stdout);

    /* REM : wait(NULL); EST SANS EFFET ICI CAR PAS DE FILS */
    sleep(2);
  }

  printf("Consommateur (pid=%d) : Fin du processus consommateur...\n", getpid());
  fflush(stdout);
  return(EXIT_SUCCESS);
}
