/*----------------------------------------------------------------------*/
/* Fichier : tpi2ldd1pingpongtubes.c					*/
/* Auteur  : D. Duvivier - LIL/ULCO					*/
/* Date    : 22/09/09							*/
/* Objectif: TP N3 M2/I2L Systme Rseaux Scurit			*/
/*	     Test de deux tubes et de la primitive "fork()" pour	*/
/*	     implementer une synchronisation de type ping-pong.		*/
/*	     Ceci est realisable avec un seul tube utilise en alternance*/
/*	     mais attention aux blocage lors du "changement de sens"	*/
/*	     (pere->fils, fils->pere) de transit des messages dans le	*/
/*	     tube !!!							*/
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>

#define TAILLE_BUF	4 /* NE PAS MODIFIER : Taille du buffer utilise pour les E/S */
#define NB_ACCES	10 /* Nombre de lectures/ecritures */

/* Variables globales -> a viter/limiter normalement (:-(!!!) */
int i; /* indice de boucle */
int pid; /* pid du fils ou 0 */
char buffer[TAILLE_BUF+1]; /* Rem : +1 pour le \0 en fin de chaine */
int tping[2], tpong[2]; /* Un tube ping et un tube pong */
int ncar; /* Nombre de caracteres effectivement lus/ecrits */

int main(void) {
  puts("Lancement du programme...");
  /* Les tubes DOIVENT etre cree AVANT le fork afin que le pere et le fils */
  /* obtiennent les memes descripteurs de fichiers et puissent alors */
  /* communiquer par leur intermediaire */
  if ((pipe(tping) < 0) || (pipe(tpong) < 0)) {
    perror("echec creation tube");
    exit(EXIT_FAILURE);
  }
  pid = fork();
  if (pid < 0) {
    perror("echec fork");
    exit(EXIT_FAILURE);
  }
  else if (pid > 0) {
    /* Code du pre */
    puts("Code du pre/producteur...");
    printf("Le pid du fils est %d\n", pid); fflush(stdout);
    /* Le pere va uniquement ecrire dans tping -> il FAUT fermer tping[0] */
    close(tping[0]);
    /* Le pere va uniquement lire dans tpong -> il FAUT fermer tpong[1] */
    close(tpong[1]);
    puts("Processus pre : Tube tping par ! Tube tpong par !");
    fflush(stdout);
    sleep(2);
    /* PING-PONG */
    for (i=0; i<NB_ACCES; i++) {
      puts("Pere : PING ecrit dans le tube tping..."); fflush(stdout);
      ncar = write(tping[1], "PING", TAILLE_BUF);
      if (ncar != TAILLE_BUF) {
        perror("echec lors de l'ecriture");
        exit(EXIT_FAILURE);
      }
      sleep(1);
      puts("Pere : attend PONG depuis le tube tpong..."); fflush(stdout);
      ncar = read(tpong[0], buffer, TAILLE_BUF);
      if (ncar != TAILLE_BUF) {
        perror("echec lors de la lecture");
        exit(EXIT_FAILURE);
      }
      buffer[ncar] = 0; /* Terminaison de chaine \0 par securite */
      printf("Pere : %s lu depuis le tube tpong\n", buffer);
      fflush(stdout);
      sleep(1);
    } 
    puts("Pere : fin du ping-pong"); fflush(stdout);
    /* Fermer les descripteurs encore ouverts */
    close(tping[1]);
    close(tpong[0]);
    puts("Fin du processus pre..."); fflush(stdout);
  }
  else {
    /* Code du fils */
    puts("Code du fils/consommateur...");
    /* Le fils va uniquement ecrire dans tpong -> il FAUT fermer tpong[0] */
    close(tpong[0]);
    /* Le fils va uniquement lire dans tping -> il FAUT fermer tping[1] */
    close(tping[1]);
    puts("Processus fils : Tube tping par ! Tube tpong par !");
    printf("Le fils se met en attente du processus pere %d...\n", getppid());
    fflush(stdout);
    sleep(2);
    /* PING-PONG */
    for (i=0; i<NB_ACCES; i++) {
      puts("Fils : attend PING depuis le tube tping..."); fflush(stdout);
      ncar = read(tping[0], buffer, TAILLE_BUF);
      if (ncar != TAILLE_BUF) {
        perror("echec lors de la lecture");
        exit(EXIT_FAILURE);
      }
      buffer[ncar] = 0; /* Terminaison de chaine \0 par securite */
      printf("Fils : %s lu depuis le tube tping\n", buffer);
      fflush(stdout);
      sleep(1);
      puts("Fils : PONG ecrit dans le tube tpong..."); fflush(stdout);
      ncar = write(tpong[1], "PONG", TAILLE_BUF);
      if (ncar != TAILLE_BUF) {
        perror("echec lors de l'ecriture");
        exit(EXIT_FAILURE);
      }
      sleep(1);
    } 
    puts("Fils : fin du ping-pong"); fflush(stdout);
    /* Fermer les descripteurs encore ouverts */
    close(tpong[1]);
    close(tping[0]);
    puts("Fin du processus fils..."); fflush(stdout);
  }
  /* Le pre et le fils vont passer ici en fin de programme sauf erreur */
  return(EXIT_SUCCESS);
}
