/*----------------------------------------------------------------------*/
/* Fichier : tpi2ldd1simpletn.c                                         */
/* Auteur  : D. Duvivier - LIL/ULCO                                     */
/* Date    : 22/09/09                                                   */
/* Objectif: TP N3 M2/I2L Systme Rseaux Scurit                     */
/*           Test d'un tube nomm et de la primitive "fork()"           */
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>

#define TAILLE_BUF      75 /* Taille du buffer sans le \0 */

/* Variables globales -> a viter/limiter normalement (!) */
int pid;
int i; /* indice de boucle */
char *nom_tube="tube1";
FILE *fichier;
int fd;
char buffer[TAILLE_BUF+1]; /* Rem : +1 pour le \0 */

int main(void) {
  puts("Lancement du programme..."); fflush(stdout);

  /* Appeler une seule fois cette commande normalement : */
  mkfifo(nom_tube, 0644);
  /* Pour simplifier ce programme la commande ci-dessus est systematiquement*/
  /* invoquee, meme si le tube nomme existe deja ! */
  
  pid = fork();
  if (pid < 0) {
    perror("echec fork");
    exit(EXIT_FAILURE);
  }
  else if (pid > 0) {
    /* Code du pre */
    puts("Code du pre...");
    printf("Le pid du fils est %d\n", pid);
    fflush(stdout);
    /* Ouverture en criture du tube en utilisant les primitives */
    /* de bas niveau, non bufferisees : */
    fd = open(nom_tube, O_WRONLY);
    /* Mise en place de fonctions de haut-niveau a partir du descripteur */
    /* precedemment obtenu afin de pouvoir manipuler le tube au niveau des */
    /* fonctions de la bibliotheque standard du c (printf, puts ...) */
    fichier = fdopen(fd, "w");
    for (i = 0; i < 100; i++) {
      fputs("test tube test tube test tube test tube test tube ", fichier);
      /* Remarque : sans le fflush() ci-dessous, les informations sont */
      /* bufferisees par le systeme, ce qui est plus efficace car minimise */
      /* les E/S, mais attention si l'on synchronise des processus en */
      /* utilisant ces primitives : il faut en tenir compte ! */
      /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
      /* ESSAYEZ EN COMMENTANT LA LIGNE SUIVANTE POUR VOUS CONVAINCRE DE */
      /* LA DIFFERENCE (avec/sans buffer lors de l'ECRITURE dans le tube)*/
      /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
      fflush(fichier);
      usleep(100000); /* Temporisation de 1/10 eme de seconde */     
    }
    fclose(fichier);
    close(fd);
    puts("Le pere attend le fils..."); fflush(stdout);
    /* Ceci evite les affichages desagreables apres l'affichage du prompt */
    /* du shell (qui ne fait qu'attendre le pere et pas le fils :-() */
    /* (essayez de commenter la ligne suivante, vous verrez ! */
    wait(NULL);
    puts("Fin du processus pre..."); fflush(stdout);
  }
  else {
    /* Code du fils */
    puts("Code du fils...");
    puts("Le fils se met en attente...");
    fflush(stdout);
    sleep(1);
    puts("Fin d'attente du fils...");
    fflush(stdout);
    /* Ouverture en lecture du le tube en utilisant les primitives */
    /* de bas niveau, non bufferisees : */
    fd = open(nom_tube, O_RDONLY);
    /* Mise en place de fonctions de haut-niveau a partir du descripteur */
    /* precedemment obtenu afin de pouvoir manipuler le tube au niveau des */
    /* fonctions de la bibliotheque standard du c (printf, puts ...) */
    fichier = fdopen(fd, "r");
    i = 0;
    /* Ci-dessous le contenu du tube est recupere par "paquets" de */
    /* TAILLE_BUF caracteres, ce qui explique l'affichage "curieux" */
    while (fgets(buffer, TAILLE_BUF, fichier) != NULL) {
      i++;
      printf("%02d %s\n", i, buffer); fflush(stdout);
    }
    fclose(fichier);
    close(fd);
    puts("Fin du processus fils..."); fflush(stdout);
  }
  /* Le pre et le fils vont passer ici en fin de programme sauf erreur */
  return(EXIT_SUCCESS);
}
