/*----------------------------------------------------------------------*/
/* Fichier : producteurshm.c                                            */
/* Auteur  : D. Duvivier - LIL/ULCO                                     */
/* Date    : 09/10/06                                                   */
/* Objectif: TP N1 (correction) M2/I2L Systme Rseaux Scurit        */
/*           Test d'utilisation d'un smaphore entre deux               */
/*           processus "lourds"                                         */
/*           Ce programme est le producteur de donnes.                 */
/*           Il se charge de lancer le consommateur de donnes.         */
/*----------------------------------------------------------------------*/
/* Remarque : les appels "fflush(stdout)" ne servent  rien si ce n'est */
/* pour s'assurer que les messages soient bien affichs immdiatement   */
/* en vitant les dlais ds  la buffrisation des e/s.                */
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>

#define  NB_SEMAPHORES 1

/* Variables globales -> a viter/limiter normalement (!) */
int pid;        /* Pid du processus fils (i.e. le consommateur) */
int semid = -1; /* Indentificateur du smaphore partag */
                /* Commun au pre et au fils */

/* Acquisition de NB_SEMAPHORES smaphore(s) (LM36) */
/* ATTENTION : le descripteur associ est retourn dans la variable */
/* globale/statique "semid" */
int sem_init(char *filename, char project_id) {
  key_t k;

  semid = semget(k = ftok(filename, project_id), NB_SEMAPHORES, 0600);
  if (semid == -1) {
    /* Premier appel -> acquisition du tableau de smaphores */
    semid = semget(k = ftok(filename, project_id), NB_SEMAPHORES, 0600 | IPC_CREAT);
    if (semid <= -1) {
      /* Erreur lors de l'acquisition */
      perror("Impossible d'obtenir des smaphores");
      exit(EXIT_FAILURE);
    }
    else {
      /* OK : on a des smaphores ! */
      printf("Processus (pid = %d) : Tableau de smaphores allou correctement (semid=%d)\n", getpid(), semid);
      return (1); /* Signale nouveau tableau de smaphores  l'appelant */
    }
  }
  else {
    /* Le tableau de smaphores existe dj -> le rutiliser ! */
    printf("Processus (pid = %d) : Tableau de smaphores r-utilis (semid=%d)\n", getpid(), semid);
    return (0); /* Signale r-utilisation des smaphores  l'appelant */
  }
}


/* Fixe la valeur initiale du smaphore */
void sem_setvalue(int sem, int val) {
#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
       /* l'union semun est dfinie par l'inclusion de <sys/sem.h> */
       union  semun arg;
#else
       /* d'aprs X/OPEN il faut la dfinir nous-mmes */
       union semun {
             int val;                  /* valeur pour SETVAL */
             struct semid_ds *buf;     /* buffer pour IPC_STAT, IPC_SET */
             unsigned short *array;    /* table  pour GETALL, SETALL */
                                       /* Spcificit Linux : */
             struct seminfo *__buf;    /* buffer pour IPC_INFO */
       } arg;
#endif

  arg.val = val; /* Fixe la valeur initiale du smaphore */
  if (semctl(semid, sem, SETVAL, arg) <= -1) {
    perror("impossible de fixer la valeur initiale du smaphore");
    exit(EXIT_FAILURE);
  }
}

/* P(smaphore) */
int sem_askresource (int sem) {
  int r;
  struct sembuf s;
  s.sem_num = sem;
  s.sem_op = -1;
  s.sem_flg = 0;
  if ((r = semop(semid, &s, 1)) <= -1) {
    perror("Impossible de rserver la ressource");
    exit(EXIT_FAILURE);
  }
  return r;
}

/* V(smaphore) */
int sem_releaseresource (int sem) {
  int r;
  struct sembuf s;
  s.sem_num = sem;
  s.sem_op = 1;
  s.sem_flg = 0;
  if ((r = semop(semid, &s, 1)) <= -1) {
    perror("Impossible de librer la ressource");
    exit(EXIT_FAILURE);
  }
  return r;
}

/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU PERE                                  */
/* Le pid du processus fils est pass en paramtre.                     */
/************************************************************************/
void ProducteurCodeDuPere(int pid) {
  int i; /* Indice de boucle */
  int code;


  printf("Producteur (pid = %d) : Lancement du code du processus pre...\n",
         getpid());
  printf("Producteur (pid = %d) : Le pid du fils est %d\n", getpid(), pid);
  puts("CODE DU PERE A FAIRE");

  printf("Producteur (pid = %d) : Le pre ??? le tableau de smaphores identifi par (semid=%d)...\n", getpid(), semid);
  fflush(stdout);


  printf("Producteur (pid = %d) : Le pre se met en attente pour 30 secondes...\n", getpid());
  fflush(stdout);
  sleep(30); /* Simule un traitement */


  /* Une dernire fois nous attendons la fin du processus fils */
  /* (si ncessaire) : */
  printf("Producteur (pid = %d) : Attente de la fin du processus fils...\n", getpid());
  fflush(stdout);
  wait(NULL);

  printf("Producteur (pid = %d) : Fin du processus pre !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU FILS                                  */
/* Ce processus est un "clone" (i.e. un fils obtenu via fork)           */
/* du processus producteur.                                             */
/************************************************************************/
void ProducteurCodeDuFils(void) {

  printf("Producteur (pid = %d) : Lancement du code du processus fils...\n",
         getpid());
  fflush(stdout);


  printf("Producteur (pid = %d) : Le fils se met en attente pour 15 secondes...\n", getpid());
  fflush(stdout);

  sleep(15);



  printf("Producteur (pid = %d) : Fin du processus fils !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* PROGRAMME PRINCIPAL : CODE DU PROCESSUS PRODUCTEUR (PERE & FILS)     */
/************************************************************************/
int main(int argc, char *argv[]) {
  printf("Producteur (pid = %d) : Lancement du programme producteur...\n", getpid());

  printf("Producteur (pid = %d) : Lancement du programme consommateur  partir du producteur...\n", getpid());

  /* On utilise le nom de l'excutable et le numro de projet (ici "1")*/
  /* pour calculer une clef via ftok et obtenir un descripteur de */
  /* tableau de smaphores unique */
  if (!sem_init(argv[0], 1)) {
    printf("Producteur (pid = %d) : Le tableau de smaphores existe dj...\n", getpid());
    /* inutile de faire exit(EXIT_FAILURE) ici ce n'est pas une erreur*/
  }

  /* En ce point nous allons lancer le consommateur  partir du */
  /* producteur en effectuant un fork() */
  /* ATTENTION : par soucis de concision, tous les codes de retour et */
  /* toutes le prcautions ncessaires  l'criture d'un programme complet*/
  /* et "robuste" n'ont pas t ajoute  ce programme */
  /* -> allez voir dans les pages de man (...) la documentation des */
  /* fonctions utilises et les prcautions  mettre en oeuvre */
  /* en guise d'exercice ! */

  pid = fork(); /* On clone le pre via fork */
  if (pid < 0) {
    fprintf(stderr, "Producteur (pid = %d) : L'erreur '%s' s'est produite lors de l'appel  fork()\n",
              getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }
  else if (pid > 0) {
    /*  Code du pre */
    ProducteurCodeDuPere(pid);
  }
  else {
    /* Code du fils */
    ProducteurCodeDuFils();
  }

  /* QUESTION SUBSIDIAIRE : QUI (pre et/ou fils) VA EXECUTER CETTE */
  /* PARTIE DU CODE EN FIN D'EXECUTION DE(S) PROGRAMME(S) */

  printf("Producteur/consommateur ? Fin du programme producteur excut par le processus pid = %d ...\n", getpid());
  printf("Producteur/consommateur ? (le processus parent de ce processus porte le numro ppid = %d)\n", getppid());
  fflush(stdout);
  return(EXIT_SUCCESS);
}
