/*----------------------------------------------------------------------*/
/* Fichier : tpi2ldd1pinpongsig.c					*/
/* Auteur  : D. Duvivier - LIL/ULCO					*/
/* Date    : 27/09/09							*/
/* Objectif: TP N3 (correction) M2/I2L Systme Rseaux Scurit	*/
/*	     Test de deux signaux entre deux processus "lourds"		*/
/*	     pour realiser une synchronisation de type "ping-pong"	*/
/*	     (synchronisation de deux processus en alternance).		*/
/*----------------------------------------------------------------------*/
/* EXERCICE : faites la meme chose en utilisant deux programmes		*/
/* executables distincts (un programme producteur/maitre/ping et un	*/
/* programme consommateur/esclave/pong), vous pouvez vous inspirer des	*/
/* programmes producteursig.c et consommateursig.c. Vous remarquerez que*/
/* ces programmes donnent un autre exemple de gestion des signaux ou une*/
/* partie de la gestion est effectuee par le programme principal et une	*/
/* autre dans le gestionnaire de signal.				*/
/*----------------------------------------------------------------------*/
/* Remarque : les appels "fflush(stdout)" ne servent  rien si ce n'est	*/
/* pour s'assurer que les messages soient bien affichs aussi rapidement*/
/* que possible en vitant les dlais ds  la buffrisation des e/s.	*/
/*----------------------------------------------------------------------*/
/* ATTENTION : par soucis de concision, tous les codes de retour et	*/
/* toutes le prcautions ncessaires  l'criture d'un programme complet*/
/* et "robuste" n'ont pas t ajoute  ce programme			*/
/* -> allez voir dans les pages de man (...) la documentation des	*/
/*    fonctions utilises et les prcautions  mettre en oeuvre		*/
/*    en guise d'exercice !						*/
/*----------------------------------------------------------------------*/
/* ATTENTION : ceci est un exemple de programmation "incorrecte" :	*/
/* elle offre l'avantage de montrer comment gerer INTEGRALEMENT la	*/
/* synchronisation via deux signaux dans les handlers (hormis le premier*/
/* envoi de signal pour amorcer le ping-pong)				*/
/* MAIS IL EST HORS DE QUESTION D'EFFECTUER DES ENTREES/SORTIES		*/
/* (ou de maniere tres controlees et non bufferises)			*/
/* OU DES TEMPORISATIONS (via sleep(), usleep()...) DANS UN handler !!!	*/
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>

/* Variables globales -> a viter/limiter normalement (!) */
int pid; /* Pid du processus fils (i.e. le pong) ou 0 */


/* Gestion des signaux pour le pere */
void signal_handler_ping(int signal) {
  static int ipong = 1; /* Nombre de "pongs" */
  switch(signal) {
  case SIGUSR2:
    printf("Ping (pid = %d) : Signal SIGUSR2 intercept !\n", getpid());
    sleep(1); /* Simule un traitement */
    ipong++;
    printf("Ping (pid = %d) : Emet PING %d via SIGUSR1 destin au pong\n",
           getpid(), ipong);
    fflush(stdout);
    kill(pid, SIGUSR1); /* Envoit le signal SIGUSR1 au fils */
  break;
  case SIGHUP:
    printf("Ping (pid = %d) : Signal SIGHUP intercept !\n", getpid());
    fflush(stdout);
  break;
  }
}


/* Gestion des signaux pour le fils */
/* (la synchronisation est totalement gre par cette fonction */
void signal_handler_pong(int signal) {
  static int iping = 0; /* Compteur de "pings" */
  switch(signal) {
  case SIGUSR1:
    printf("Pong (pid=%d) : Signal SIGUSR1 intercept !\n", getpid());
    sleep(1); /* Simule un traitement */
    iping++;
    printf("Pong (pid=%d) : Emet PONG %d via SIGUSR2 destin au ping\n",
           getpid(), iping);
    fflush(stdout);
    kill(getppid(), SIGUSR2); /* Envoit le signal SIGUSR2 au pre */
  break;
  case SIGHUP:
    puts("Signal SIGHUP intercept par notre handler_pong");
    fflush(stdout);
  break;
  }
}


/************************************************************************/
/* CODE DU PERE	: Le pid du processus fils est pass en paramtre.	*/
/************************************************************************/
void CodeDuPere(int pid) {
  int i; /* Indice de boucle */
  /* int code; */
  struct sigaction action;

  printf("Ping (pid = %d) : Lancement du code du processus ping...\n",
         getpid());
  printf("Ping (pid = %d) : Le pid du fils est %d\n", getpid(), pid);
  fflush(stdout);
  /* Seul le pre intercepte ces signaux ici,  la place de signal(), */
  /* on utilise cette fois une structure sigaction qui permet plus */
  /* de possibilits dans la gestions des signaux ... */
  /* les lignes suivantes remplacent en fait */
  /* signal(SIGUSR2, signal_handler_ping); */
  /* par un appel systeme un peu plus "elabore" : sigaction() */
  action.sa_handler=signal_handler_ping;
  sigemptyset(&(action.sa_mask));
  action.sa_flags = 0; /* Remplacer 0 par SA_ONESHOT pour une seule interception du signal */
  if (sigaction(SIGUSR2, &action, NULL) != 0) {
    fprintf(stderr, "Ping (pid = %d) : Impossible de mettre en place le gestionnaire de signaux : %s\n", getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }
  /* Tout est en place : les processus sont lances, les gestionnaires de */
  /* signaux sont en place, nous generons simplement une petite attente */
  sleep(3);
  printf("Ping (pid = %d) : Lancement du ping pong...\n", getpid());
  fflush(stdout);

  /* On amorce le ping-pong en envoyant le premier signal au fils */
  printf("Ping (pid = %d) : Lancement : Emet PING %d via SIGUSR1 destin au pong\n", getpid(), 0);
  fflush(stdout);
  kill(pid, SIGUSR1); /* Envoit le signal SIGUSR1 au fils */
    
  /* La boucle ci-dessous simule un traitement (principal) */
  /* interatif effectue par le pere (ce traitement est interrompu */
  /* periodiquement par le traitement des signaux */
  /* ATTENTION au cas particulier ici : le sleep dans le gestionnaire */
  /* de signaux vient perturber le sleep(10) ci-dessous qui est interrompu*/
  /* prematurement, ce qui impose l'utilisation d'une boucle autour du */
  /* sleep(), mais en theorie UN SEUL sleep(300) pourrait suffire a */
  /* simuler un calcul long interrompu periodiquement et A PLUSIEURS */
  /* reprises par un signal, c'est un cas particulier du a l'utilisation*/
  /* de sleep dans le handler ! */
  for (i = 1; i <= 12; i++) {
    sleep(10);
    printf("Traitement effectue par Ping (pid = %d) : iteration %d\n",
           getpid(), i);
    fflush(stdout);
  }

  /* Une dernire fois nous attendons la fin du processus fils */
  /* (si ncessaire) via un appel a wait() : */
  printf("Ping (pid = %d) : Attente de la fin du processus fils...\n",
         getpid());
  fflush(stdout);
  wait(NULL);

  printf("Ping (pid = %d) : Fin du processus pre !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* CODE DU FILS								*/
/************************************************************************/
void CodeDuFils(void) {
  int i = 0; /* Indice de boucle */

  printf("Pong (pid=%d) : Lancement du processus pong...\n", getpid());
  fflush(stdout);

  signal(SIGHUP, signal_handler_pong);
  signal(SIGUSR1, signal_handler_pong);

  /* Ci-dessous la boucle sur sleep() simule le traitement normal */
  /* du processus fils */
  /* Ce traitement "normal" est priodiquement interrompu par le pre */
  /* et signal_handler_pong intercepte ce signal, traite le signal et */
  /* renvoit un signal au pre. */
  /* Ici le programme principal ne s'occupe absolument pas de la gestion*/
  /* des signaux (qui est totalement gre par signal_handler_pong() */
  /* ceci est possible car il n'y a pas de variable partage (...) */
  /* entre signal_handler_pong() et main(), sinon attention il faut grer */
  /* ces partages correctement via des sections critiques ... */
  /* ATTENTION au cas particulier ici : le sleep dans le gestionnaire */
  /* de signaux vient perturber le sleep(10) ci-dessous qui est interrompu*/
  /* prematurement, ce qui impose l'utilisation d'une boucle autour du */
  /* sleep(), mais en theorie UN SEUL sleep(300) pourrait suffire a */
  /* simuler un calcul long interrompu periodiquement et A PLUSIEURS */
  /* reprises par un signal, c'est un cas particulier du a l'utilisation*/
  /* de sleep dans le handler ! */
  for (i = 1; i <= 12; i++) {
    sleep(10);
    printf("Traitement effectue par Pong (pid = %d) : iteration %d\n",
           getpid(), i);
    fflush(stdout);
  }

  /* REM : wait(NULL); EST SANS EFFET ICI CAR PAS DE FILS */

  printf("Pong (pid=%d) : Fin du processus pong...\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* PROGRAMME PRINCIPAL : CODE DU PROCESSUS PRODUCTEUR (PERE & FILS)     */
/************************************************************************/
int main(void) {
  printf("Ping (pid = %d) : Lancement du programme ping...\n", getpid());
  pid = fork(); /* On clone le pre via fork */
  if (pid < 0) {
    fprintf(stderr, "Ping (pid = %d) : L'erreur '%s' s'est produite lors de l'appel  fork()\n",
              getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }
  else if (pid > 0) {
    /*  Code du pre */
    CodeDuPere(pid);
  }
  else {
    /* Code du fils */
    CodeDuFils();
  }
  printf("Fin du programme excut par le processus pid = %d ...\n", getpid());
  printf("(le processus parent de ce processus porte le numro ppid = %d)\n",
         getppid());
  fflush(stdout);
  return(EXIT_SUCCESS);
}
