/*----------------------------------------------------------------------*/
/* Fichier : producteur2sem.c						*/
/* Auteur  : D. Duvivier - LIL/ULCO					*/
/* Date    : 14/10/08							*/
/* Objectif: TP N3 (correction) M2/I2L Systme Rseaux Scurit	*/
/*	     Test d'utilisation de 2 smaphores entre deux		*/
/*	     processus "lourds" pour effectuer une synchronisation en	*/
/*	     alternance (Ping-Pong) entre un processus producteur et un */
/*	     processus consommateur.					*/
/*	     Ce programme contient le producteur et le consommateur de	*/
/*	     donnes.							*/
/*----------------------------------------------------------------------*/
/* CETTE VERSION EST UNE CORRECTION DE LA SYNCHRONISATION "PING PONG"	*/
/* REALISEE DE MANIERE INCORRECTE AVEC UN SEUL SEMAPHORE UTILISE SELON	*/
/* LE SCHEMA DE PROTECTION D'UNE SECTION CRITIQUE			*/
/* -> avec un seul smaphore on va simplement garantir de ne pas avoir	*/
/* un affichage "ping" et un affichage "pong" en mme temps (mutex)	*/
/* mais l'alternance ping -> pong -> ping ... n'est ABSOLUMENT pas	*/
/* garantie (il suffit d'allonger la dure d'un sleep de l'un des	*/
/* processus pour dsynchroniser le tout -> Inacceptable		*/
/* Le prsent programme utilise 2 smaphores pour garantir cette alter- */
/* nance, et par le biais de l'utilisation de sem_init nous pouvons	*/
/* garantir que la squence commencera par un ping (matrialisant la	*/
/* la production d'un message) avant un pong (matrialisant la		*/
/* consommation ou le traitement de ce message).			*/
/*----------------------------------------------------------------------*/
/* Remarque : les appels "fflush(stdout)" ne servent  rien si ce n'est	*/
/* pour s'assurer que les messages soient bien affichs immdiatement	*/
/* en vitant les dlais ds  la buffrisation des e/s.		*/
/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>

#define  NB_SEMAPHORES 2

/* Variables globales -> a viter/limiter normalement (!) */
int pid;        /* Pid du processus fils (i.e. le consommateur) */
int semid = -1; /* Identificateur du tableau de smaphores partag */
		/* Commun au pre et au fils */

/* Acquisition de NB_SEMAPHORES smaphore(s) (LM36) */
/* ATTENTION : le descripteur associ est retourn dans la variable */
/* globale/statique "semid" */
int sem_init(char *filename, char project_id) {
  key_t k;

  if (semid == -1) {
    /* Accder au tableau de smaphores s'il existe dj */
    semid = semget(k = ftok(filename, project_id), NB_SEMAPHORES, 0600);
    if (semid == -1) {
      /* Premier appel -> acquisition du tableau de smaphores */
      semid = semget(k = ftok(filename, project_id), NB_SEMAPHORES, 0600 | IPC_CREAT);
      if (semid <= -1) {
        /* Erreur lors de l'acquisition */
        perror("Impossible d'obtenir des smaphores");
        exit(EXIT_FAILURE);
      }
      else {
        /* OK : on a des smaphores ! */
        printf("Processus (pid = %d) : Tableau de smaphores allou correctement (semid=%d)\n", getpid(), semid);
        return (1); /* Signale nouveau tableau de smaphores  l'appelant */
      }
    }
    else {
      /* Le tableau de smaphores existe dj et c'est le premier appel  sem_init -> le rutiliser ! */
      printf("Processus (pid = %d) : Tableau de smaphores r-utilis (semid=%d)\n", getpid(), semid);
      return (0); /* Signale r-utilisation des smaphores  l'appelant */
    }
  }
  else {
    /* Le tableau de smaphores existe dj et ce n'est pas le premier appel  sem_init -> le rutiliser ! */
    printf("Processus (pid = %d) : Tableau de smaphores r-utilis (semid=%d)\n", getpid(), semid);
    return (-1); /* Signale r-utilisation des smaphores  l'appelant */
                 /* et appels multiples de sem_init */
  }
}

/* Suppression du tableau de smaphores dont l'id est stock dans la*/
/* variable globale semid. semid est reinitialise a -1 par cette procedure */
void sem_destroy(void) {
  semctl(semid, 0, IPC_RMID, 0);
  semid = -1; /* Scurit : indique tableau de semaphores non allou */
}

/* Fixe la valeur initiale du smaphore */
void sem_setvalue(int sem, int val) {
#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
       /* l'union semun est dfinie par l'inclusion de <sys/sem.h> */
       union  semun arg;
#else
       /* d'aprs X/OPEN il faut la dfinir nous-mmes */
       union semun {
             int val;                  /* valeur pour SETVAL */
             struct semid_ds *buf;     /* buffer pour IPC_STAT, IPC_SET */
             unsigned short *array;    /* table  pour GETALL, SETALL */
                                       /* Spcificit Linux : */
             struct seminfo *__buf;    /* buffer pour IPC_INFO */
       } arg;
#endif

  arg.val = val; /* Fixe la valeur initiale du smaphore */
  if (semctl(semid, sem, SETVAL, arg) <= -1) {
    perror("impossible de fixer la valeur initiale du smaphore");
    exit(EXIT_FAILURE);
  }
}

/* P(smaphore) */
int sem_askresource (int sem) {
  int r;
  struct sembuf s;
  s.sem_num = sem;
  s.sem_op = -1;
  s.sem_flg = 0;
  if ((r = semop(semid, &s, 1)) <= -1) {
    perror("Impossible de rserver la ressource");
    exit(EXIT_FAILURE);
  }
  return r;
}

/* V(smaphore) */
int sem_releaseresource (int sem) {
  int r;
  struct sembuf s;
  s.sem_num = sem;
  s.sem_op = 1;
  s.sem_flg = 0;
  if ((r = semop(semid, &s, 1)) <= -1) {
    perror("Impossible de librer la ressource");
    exit(EXIT_FAILURE);
  }
  return r;
}

/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU PERE					*/
/* Le pid du processus fils est pass en paramtre.			*/
/************************************************************************/
void ProducteurCodeDuPere(int pid) {
  int i; /* Indice de boucle */

  printf("Producteur (pid = %d) : Lancement du code du processus pre...\n",
         getpid());
  printf("Producteur (pid = %d) : Le pid du fils est %d\n", getpid(), pid);
  printf("Producteur (pid = %d) : Lancement du ping pong...\n",
        getpid());
  fflush(stdout);

  for (i = 1; i <= 10; i++) {
    printf("Producteur (pid = %d) : Demande d'accs  la ressource critique\n", getpid());
    fflush(stdout);

    sem_askresource(0); /* P(semaphore[0]) */

    printf("Producteur (pid = %d) : Accs (PING %d)  la ressource critique\n", getpid(), i);
    fflush(stdout);

    sleep(1); /* Une seconde en ressource critique */

    printf("Producteur (pid = %d) : Libre le consommateur...\n", getpid());    fflush(stdout);
    sem_releaseresource(1); /* V(semaphore[1]) */

    printf("Producteur (pid = %d) : Hors de la ressource critique...\n", getpid());    fflush(stdout);

    sleep(1); /* Une seconde hors de ressource critique */
  }

  /* Une dernire fois nous attendons la fin du processus fils */
  /* (si ncessaire) : */
  printf("Producteur (pid = %d) : Attente de la fin du processus fils...\n", getpid());
  fflush(stdout);
  wait(NULL);

  printf("Producteur (pid = %d) : Fin du processus pre !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* PROCESSUS PRODUCTEUR : CODE DU FILS					*/
/* Ce processus est un "clone" (i.e. un fils obtenu via fork)		*/
/* du processus producteur.						*/
/************************************************************************/
void ProducteurCodeDuFils(void) {
  int i; /* Indice de boucle */
  printf("Producteur (pid = %d) : Lancement du code du processus fils...\n",
         getpid());
  fflush(stdout);


  printf("Producteur (pid = %d) : Lancement du ping pong...\n",
        getpid());
  fflush(stdout);

  for (i = 1; i <= 10; i++) {
    printf("Producteur (pid = %d) : Demande d'accs  la ressource critique\n", getpid());
    fflush(stdout);

    sem_askresource(1); /* P(semaphore[1]) */

    printf("Producteur (pid = %d) : Accs (PONG %d)  la ressource critique\n", getpid(), i);
    fflush(stdout);

    sleep(1); /* Une seconde en ressource critique */

    printf("Producteur (pid = %d) : Libre le producteur...\n", getpid());    fflush(stdout);
    sem_releaseresource(0); /* V(semaphore[0]) */

    printf("Producteur (pid = %d) : Hors de la ressource critique...\n", getpid());    fflush(stdout);

    /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
    /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
    /* OK : fonctionne quelle que soit la dure de temporisation                   */
    /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
    /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
    sleep(4); /* Quatre secondes hors de ressource critique */
  }


  printf("Producteur (pid = %d) : Fin du processus fils !\n", getpid());
  fflush(stdout);
}

/************************************************************************/
/* PROGRAMME PRINCIPAL : CODE DU PROCESSUS PRODUCTEUR (PERE & FILS)     */
/************************************************************************/
int main(int argc, char *argv[]) {
  printf("Producteur (pid = %d) : Lancement du programme producteur...\n", getpid());

  printf("Producteur (pid = %d) : Lancement du programme consommateur  partir du producteur...\n", getpid());

  /* On utilise le nom de l'excutable et le numro de projet (ici "1")*/
  /* pour calculer une clef via ftok et obtenir un descripteur de */
  /* tableau de smaphores unique */
  if (!sem_init(argv[0], 1)) {
    printf("Producteur (pid = %d) : Le tableau de smaphores existe dj...\n", getpid());
    /* inutile de faire exit(EXIT_FAILURE) ici ce n'est pas une erreur*/
  }

  /* On fixe la valeur initiale du smaphore N0  la valeur 1 */
  /* (ping du producteur ds le lancement) */
  sem_setvalue(0, 1);

  /* On fixe la valeur initiale du smaphore N1  la valeur 0 */
  /* (producteur (pong) bloqu ds le lancement) */
  sem_setvalue(1, 0);

  /* En ce point nous allons lancer le consommateur  partir du */
  /* producteur en effectuant un fork() */
  /* ATTENTION : par soucis de concision, tous les codes de retour et */
  /* toutes le prcautions ncessaires  l'criture d'un programme complet*/
  /* et "robuste" n'ont pas t ajoute  ce programme */
  /* -> allez voir dans les pages de man (...) la documentation des */
  /* fonctions utilises et les prcautions  mettre en oeuvre */
  /* en guise d'exercice ! */

  pid = fork(); /* On clone le pre via fork */
  if (pid < 0) {
    fprintf(stderr, "Producteur (pid = %d) : L'erreur '%s' s'est produite lors de l'appel  fork()\n",
              getpid(), strerror(errno));
    exit(EXIT_FAILURE);
  }
  else if (pid > 0) {
    /*  Code du pre */
    ProducteurCodeDuPere(pid);
    /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
    /* C.S.G. : Cerise Sur le Gateau : on termine proprement en attendant le fils  */
    /* avant de dtruire le tableau de smaphores.                                 */
    /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
    printf("Producteur (pid = %d) : Attente de la fin d'excution du processus fils/consommateur\n", getpid());
    fflush(stdout);
    wait(NULL);
    printf("Producteur (pid = %d) : Destruction du tableau de smaphores\n", getpid());
    fflush(stdout);
    sem_destroy();
  }
  else {
    /* Code du fils */
    ProducteurCodeDuFils();
  }

  /* QUESTION SUBSIDIAIRE : QUI (pre et/ou fils) VA EXECUTER CETTE */
  /* PARTIE DU CODE EN FIN D'EXECUTION DE(S) PROGRAMME(S) */

  printf("Producteur/consommateur ? Fin du programme producteur excut par le processus pid = %d ...\n", getpid());
  printf("Producteur/consommateur ? (le processus parent de ce processus porte le numro ppid = %d)\n", getppid());
  fflush(stdout);
  return(EXIT_SUCCESS);
}
