/*----------------------------------------------------------------------*/
/* Fichier : tstskieursteleski.c					*/
/* Auteur  : D. Duvivier - LIL/ULCO					*/
/* Date    : 21/02/08							*/
/* Objectif: TP/TD N1 L2/Info Systmes d'Exploitation			*/
/*	     Test d'une synchronisation entre processus			*/
/*	     -> modle "skieurs / tlski"				*/
/*----------------------------------------------------------------------*/
/*
#############################################################################
#This program is free software; you can redistribute it and/or modify       #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#############################################################################
 */
#include <ddtools.h>

/* Ci-dessous on utilise un cinquime smaphore en guise de variable */
/* partage entre plusieurs processus pour viter les complications */
#define normale		semaphore0
#define entree_sas	semaphore1
#define perche		semaphore2
#define protege_ecole	semaphore3
#define ecole		semaphore4

/* Nombre de processus skieurs pour la file "normale" et pour la file */
/* "ecole de ski", doivent tre > 0  */
/* S.V.P. ne dpassez pas 32000 processus au total :-)) ! */
#define NBECOLES	10
#define NBNORMALES	5

/* Nombre de perches disponibles au maximum */
#define NBPERCHES	8

/* Nombre d'accs au tklski de chaque processus, */
/* ces valeurs doivent tre > 0  */
#define NBACCESECOLES	10
#define NBACCESNORMALES	10

/* Variables globales -> a viter/limiter normalement (!) */
/* -> ICI AUCUNE !!! */

/************************************************************************/
/* CODE D'INITIALISATION SPECIFIQUE A UN EXERCICE PARTICULIER :		*/
/* A APPELER AVANT LE/LES APPEL(s) A FORK :				*/
/************************************************************************/
void CodeDInitialisation(int argc, char *argv[]) {
  DD_init(argc, argv, 5); /* Initialisation pour 5 smaphores AVANT LE FORK !!! */
  DD_sem_setvalue(normale, 1); /* Initialise smaphore normale  1 */
  DD_sem_setvalue(entree_sas, 1); /* Initialise smaphore entree_sas  1 */
  DD_sem_setvalue(perche, NBPERCHES); /* Initialise smaphore normale  NBPERCHES */
  DD_sem_setvalue(protege_ecole, 1); /* Initialise protege_ecole  1 */
  DD_sem_setvalue(ecole, 0); /* Initialise ecole  0 */
}

/************************************************************************/
/* CODE D'UN SKIEUR DE LA FILE "NORMALE".				*/
/*----------------------------------------------------------------------*/
/* "Numero" permet de distinguers les skieurs "normaux" entre-eux	*/
/************************************************************************/
void CodeDuNormale(int Numero) {
  int i; /* Indice de boucle */
  char NormaleName[18]; /* NORMALE2147483647 + \0 */

  sprintf(NormaleName, "NORMALE%-d", Numero);
  DD_pidmsg2(NormaleName, "Lancement du code du processus");
  DD_initrandom(); /* Initialiser le gnrateur alatoire */

  /* Gnre un petit dlai alatoire */
  DD_RAND_USLEEP(100000);
  /* Dbut des demande d'accs au tlski */
  for (i=0; i<NBACCESNORMALES; i++) {
    Down(normale);
    DD_pidmsg2(NormaleName, "File NORMALE bloque");
    Down(entree_sas);
    DD_pidmsg2(NormaleName, "Accs au SAS : OK !");
    Up(normale);
    DD_pidmsg2(NormaleName, "File NORMALE dbloque");
    Down(perche);

    /* Ajout pour obtenir des statistiques approximatives sur le nombre de perches disponibles */
    printf("%-12s:pid=%d: #Perches %d\n", NormaleName, getpid(), DD_sem_getvalue(perche));
    fflush(stdout);

    DD_pidmsg2(NormaleName, "Accs  une perche : OK !");
    Up(entree_sas);
    DD_pidmsg2(NormaleName, "Quitte le SAS");
    /*************************** Prendre_Perche() ***************************/
    /******************************* Monter() *******************************/
    printf("%-12s:pid=%d:Prendre perche et monter, accs N%d\n", NormaleName, getpid(), i);
    fflush(stdout);
    DD_RAND_USLEEP(100000); /* 1/10 secondes environ en section critique */
    printf("%-12s:pid=%d:Arrive sur les piste, accs N%d\n", NormaleName, getpid(), i);
    Up(perche);

    /* Ajout pour obtenir des statistiques approximatives sur le nombre de perches disponibles */
    printf("%-12s:pid=%d: #Perches %d\n", NormaleName, getpid(), DD_sem_getvalue(perche));
    fflush(stdout);

    DD_RAND_USLEEP(500000); /* 5/10 secondes environ hors de section critique */
  }
  DD_pidmsg2(NormaleName, "Fin du code du processus !");
}

/************************************************************************/
/* CODE D'UN SKIEUR DE L'ECOLE DE SKI					*/
/*----------------------------------------------------------------------*/
/* "Numero" permet de distinguers les skieurs de l'cole		*/
/************************************************************************/
void CodeDuEcole(int Numero) {
  int i = 0; /* Indice de boucle */
  char EcoleName[16]; /* Ecole2147483647 + \0 */

  sprintf(EcoleName, "ECOLE%-d", Numero);
  DD_pidmsg2(EcoleName, "Lancement du code du processus");
  DD_initrandom(); /* Initialiser le gnrateur alatoire */

  /* Gnre un petit dlai alatoire */
  DD_RAND_USLEEP(100000);
  /* Dbut des demandes d'accs au tlski */
  for (i=0; i<NBACCESECOLES; i++) {
    Down(protege_ecole);
    Incremente(ecole);
    if (DD_sem_getvalue(ecole) == 1) { 
      Down(normale); // 1er skieur de l'ecole
      DD_pidmsg2(EcoleName, "Premier skieur de l'cole bloque la file NORMALE");
    }
    Up(protege_ecole);

    Down(entree_sas);
    DD_pidmsg2(EcoleName, "Accs au SAS : OK !");

    Down(protege_ecole);
    Decremente(ecole);
    if (DD_sem_getvalue(ecole) == 0) {
      Up(normale); // Dernier ecole -> Laisser la place aux skieurs de la file NORMALE
      DD_pidmsg2(EcoleName, "Dernier skieur de l'cole dbloque la file NORMALE");
    }
    Up(protege_ecole);

    Down(perche);

    /* Ajout pour obtenir des statistiques approximatives sur le nombre de perches disponibles */
    printf("%-12s:pid=%d: #Perches %d\n", EcoleName, getpid(), DD_sem_getvalue(perche));
    fflush(stdout);

    DD_pidmsg2(EcoleName, "Accs  une perche : OK !");
    Up(entree_sas);
    DD_pidmsg2(EcoleName, "Quitte le SAS");
    /*************************** Prendre_Perche() ***************************/
    /******************************* Monter() *******************************/
    printf("%-12s:pid=%d:Prendre perche et monter, accs N%d\n", EcoleName, getpid(), i);
    fflush(stdout);
    DD_RAND_USLEEP(75000); /* 75/100 secondes environ en section critique */
    printf("%-12s:pid=%d:Arrive sur les piste, accs N%d\n", EcoleName, getpid(), i);
    Up(perche);

    /* Ajout pour obtenir des statistiques approximatives sur le nombre de perches disponibles */
    printf("%-12s:pid=%d: #Perches %d\n", EcoleName, getpid(), DD_sem_getvalue(perche));
    fflush(stdout);

    DD_RAND_USLEEP(500000); /* 5/10 secondes environ hors de section critique */
  }
  DD_pidmsg2(EcoleName, "Fin du code du processus");
}

/************************************************************************/
/* CODE DU PERE								*/
/*----------------------------------------------------------------------*/
/* De manire  tre le plus "gnrique" possible, le code du pre va	*/
/* simplement consister  lancer les rdacteurs puis les ecoles.	*/
/* Ce processus se bloquera ensuite en attente de la fin de tous les	*/
/* processus fils (ecoles et normales) via un appel  la procdure  */
/* DD_waitallchilds().							*/
/************************************************************************/
void CodeDuPere(void) {
  int i; /* Indice de boucle */
  int normalespid[NBNORMALES]; /* Pid des processus normales */
  int ecolespid[NBECOLES]; /* Pid des processus ecoles */

  DD_pidmsg2("PERE", "Lancement du code du processus...");
  /* Tout est en place, nous gnrons simplement une petite attente */
  sleep(2);
  DD_pidmsg2("PERE", "Lancement des processus NORMALES...");

  for (i=0; i<NBNORMALES; i++) {
    normalespid[i] = fork(); /* On clone le pre pour obtenir 1 normale */
    if (normalespid[i] < 0) {
      DD_piderr2("PERE", "Erreur lors de l'appel  fork()");
      exit(EXIT_FAILURE); /* fork() a echoue -> fin du programme */
    }
    else if (normalespid[i] == 0) {
      /* Code du fils : il faut lancer le producteur N "i" ... */
      CodeDuNormale(i);
      /* et ne surtout pas oublier de stopper la boucle pour */
      /* par un "break;" !!! */
      /* sinon cela va lancer des rdacteurs en cascade 1, 2, 4... !!! */
      i = -1; /* Indique boucle termine par break dans un fils */
      break; /* -> fin de boucle */
    }
    /* else/sinon (normalepid[i] > 0) */
    /* -> nous sommes dans le processus pre, dans lequel la */
    /* boucle for/pour doit se poursuivre pour lancer les normales... */
  } /* for */
  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
  /* ATTENTION LA SUITE DE CE CODE EST EXECUTEE PAR LE PERE, MAIS AUSSI PAR */
  /* CHAQUE PROCESSUS NORMALE LANCE VIA UN APPEL A fork() CI-DESSUS !!!!!!*/
  /* -> IL FAUT DISTINGUER LE PERE DES FILS/NORMALES POUR EFFECTUER LA !!!*/
  /*    SUITE DU CODE SINON "GARE AUX CASCADES DE PROCESSUS !!!!!!!!!!!!!!!!*/
  /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
  /* -> Si i >= 0 alors il s'agit du processus pre, par construction */
  if (i >= 0) {
    DD_pidmsg2("PERE", "Lancement des processus ECOLES...");
    for (i=0; i<NBECOLES; i++) {
      ecolespid[i] = fork(); /* On clone le pre pour obtenir 1 ecole */
      if (ecolespid[i] < 0) {
        DD_piderr2("PERE", "Erreur lors de l'appel  fork()");
        exit(EXIT_FAILURE); /* fork() a echoue -> fin du programme */
      }
      else if (ecolespid[i] == 0) {
        /* Code du fils : il faut lancer le ecole N "i" ... */
        CodeDuEcole(i);
        /* et ne surtout pas oublier de stopper la boucle pour */
        /* par un "break;" !!! */
        /* sinon cela va lancer des ecoles en cascade 1, 2, 4... !!! */
        i = -1; /* Indique boucle termine par break dans un fils */
        break; /* -> fin de boucle */
      }
      /* else/sinon (ecolepid[i] > 0) */
      /* -> nous sommes dans le processus pre, dans lequel la */
      /* boucle for/pour doit se poursuivre pour lancer les ecoles... */
    } /* for */
    /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
    /* ATTENTION LA SUITE DE CE CODE EST EXECUTEE PAR LE PERE, MAIS AUSSI PAR */
    /* CHAQUE PROCESSUS ECOLE LANCE VIA UN APPEL A fork() CI-DESSUS !!!!!!!!*/
    /* -> IL FAUT DISTINGUER LE PERE DES FILS/ECOLES POUR EFFECTUER LA !!!!!*/
    /*    SUITE DU CODE SINON "GARE AUX CASCADES DE PROCESSUS !!!!!!!!!!!!!!!!*/
    /*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
    /* -> Si i >= 0 alors il s'agit du processus pre, par construction */
    if (i >= 0) {
      DD_pidmsg2("PERE", "Fin du code du processus pre !");
    } /* if (i >= 0) */
    else { 
      /* sinon -> C'EST UN ECOLE -> FORCER LA FIN DU CODE */
      /* afin que la suite du code ne soit exccute que par le pre */
      exit(EXIT_SUCCESS);
    }
  } /* if (i >= 0) */
  else {
    /* sinon -> C'EST UN NORMALE -> FORCER LA FIN DU CODE */
    /* afin que la suite du code ne soit exccute que par le pre */
    exit(EXIT_SUCCESS);
  }
}

/************************************************************************/
/************************************************************************/
/* PROGRAMME PRINCIPAL : LANCEMENT DES PROCESSUS PERE ET FILS)		*/
/*----------------------------------------------------------------------*/
/* Ce programme doit garantir que le processus pre soit le dernier	*/
/* processus actif (pour appeler DD_close().				*/
/************************************************************************/
/************************************************************************/
int main(int argc, char *argv[]) {
  DD_pidmsg2("PERE", "Lancement du programme...");
  /* Initialisation(s) avant fork() */
  CodeDInitialisation(argc, argv);
  /* Code du pre */
  CodeDuPere();
  DD_pidmsg2("PERE", "Bloqu en attente de la terminaison de ses fils...");
  /* Bloquer le processus pre tant qu'au moins un fils est actif : */
  DD_waitallchilds();
  DD_close(); /* Seul le pre (dernier process actif) doit appeler DD_close() */
  /* Le pre et les fils vont passer ici en fin de programme sauf erreur */
  /* ou appel  exit() explicite dans le code des processus */
  /* QUESTION SUBSIDIAIRE : */
  /* QUI (pre et/ou fils) VA EFFECTIVEMENT EXECUTER CETTE */
  /* PARTIE DU CODE EN FIN D'EXECUTION DE(S) PROCESSUS */
  DD_pidmsg2("PEREouFILS", "Fin du programme");
  DD_pidmsg3("PEREouFILS","le processus parent porte le numro (ppid=", getppid());
  return(EXIT_SUCCESS);
}
